using System;					
using System.Windows.Forms;
using System.Threading;			// for the Thread class

namespace ThreadEventTest
{
	/// <summary>
	/// Summary description for Form1.
	/// </summary>
	public class Form1 : System.Windows.Forms.Form
	{
		private System.Windows.Forms.Button GoButton;
		private System.Windows.Forms.Button StopButton;
		private System.Windows.Forms.RichTextBox OutputBox;
		/// <summary>
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

		/// <summary>
		/// My variables
		/// </summary>

		// declare the object that's going to do the lengthy processing, and a persistent
		// thread object for the lengthy processing
		private Worker W;
		private Thread WorkerThread;

		public Form1()
		{
			//
			// Required for Windows Form Designer support
			//
			InitializeComponent();

			// App specific initializations:
			// Create the worker object at application initialization time.
			//   This is also a good time to subscribe the event-handler to the worker's
			//   event.
			// W.OnWorkerOutput is the event object itself, the thing to which we're
			//   subscribing.
			// Worker.WorkerOutputHandler is the type that represents the prototype for
			//   the event handler.  This 'new' statement creates a delegate object which
			//   encapsulates the event handler; it is that delegate which is actually
			//   subscribed to the event.
			// UpdateTextBox is the event handler, and is a method belonging to Form1.
			W = new Worker();
			W.OnWorkerOutput += new Worker.WorkerOutputHandler(UpdateTextBox);

			// Name the thread Form1 is running in for debugging purposes:
			Thread.CurrentThread.Name = "UI_Thread";
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if (components != null) 
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Windows Form Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			this.GoButton = new System.Windows.Forms.Button();
			this.StopButton = new System.Windows.Forms.Button();
			this.OutputBox = new System.Windows.Forms.RichTextBox();
			this.SuspendLayout();
			// 
			// GoButton
			// 
			this.GoButton.Location = new System.Drawing.Point(8, 8);
			this.GoButton.Name = "GoButton";
			this.GoButton.TabIndex = 0;
			this.GoButton.Text = "Go";
			this.GoButton.Click += new System.EventHandler(this.GoButton_Click);
			// 
			// StopButton
			// 
			this.StopButton.Enabled = false;
			this.StopButton.Location = new System.Drawing.Point(88, 8);
			this.StopButton.Name = "StopButton";
			this.StopButton.TabIndex = 1;
			this.StopButton.Text = "Stop";
			this.StopButton.Click += new System.EventHandler(this.StopButton_Click);
			// 
			// OutputBox
			// 
			this.OutputBox.Location = new System.Drawing.Point(0, 40);
			this.OutputBox.Name = "OutputBox";
			this.OutputBox.Size = new System.Drawing.Size(368, 632);
			this.OutputBox.TabIndex = 2;
			this.OutputBox.Text = "Click Go.";
			// 
			// Form1
			// 
			this.AutoScaleBaseSize = new System.Drawing.Size(5, 13);
			this.ClientSize = new System.Drawing.Size(368, 674);
			this.Controls.Add(this.OutputBox);
			this.Controls.Add(this.StopButton);
			this.Controls.Add(this.GoButton);
			this.Name = "Form1";
			this.Text = "Form1";
			this.ResumeLayout(false);

		}
		#endregion

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		[STAThread]
		static void Main() 
		{
			Application.Run(new Form1());
		}

		// The event handler.  The prototype--returning void, taking object and event
		// args parameters--must match the delegate defined in Worker for
		// WorkerOutputHandler.  The event handler will be initially invoked by the
		// worker thread, when it raises its OnWorkerOutput event.  Because we aren't
		// allowed to update the OutputBox control from the worker thread, the event
		// handler must re-call itself via BeginInvoke in order to switch threads.
		// Combining the thread-switching and the control-updating into one function like
		// this with an if/else structure is just convention.  We could just as well use
		// BeginInvoke to call some other function to do the actual control updating, but
		// this solution is probably cleaner and easier to maintain.
		public void UpdateTextBox(object worker, Worker.WorkerEventArgs e)
		{
			if(OutputBox.InvokeRequired) // switch threads if necessary
			{
				this.BeginInvoke(new Worker.WorkerOutputHandler(UpdateTextBox),
								 new object[]{worker,e});
			} 
			else 
			{
				OutputBox.AppendText("UpdateTextBox on thread " + 
									 Thread.CurrentThread.Name + ":\n  ");
				OutputBox.AppendText(e.status);	// put the event data into the text box.
			}
		}

		// Here we create the worker thread, put it into the background, and start it.
		// WorkerThread is a field of Form1, rather than a stack variable declared here,
		// so that it doesn't go out of scope as soon as GoButton_Click returns; we
		// don't use WorkerThread anywhere else, but it's conceivable that we might,
		// so using a persistent field is better general coding practice.
		private void GoButton_Click(object sender, System.EventArgs e)
		{
			GoButton.Enabled = false;
			StopButton.Enabled = true;
            OutputBox.Text = "GoButton_Click invoked on thread " +
							 Thread.CurrentThread.Name + "\n";
			WorkerThread = new Thread(new ThreadStart(W.Go));
			WorkerThread.Name = "Worker_Thread";
			WorkerThread.IsBackground = true;
			WorkerThread.Start();
			OutputBox.AppendText("WorkerThread.Start() has been called by thread " +
								 Thread.CurrentThread.Name + "\n");
		}

		// Call a method in the Worker object, W, to tell the thread to finish up.
		private void StopButton_Click(object sender, System.EventArgs e)
		{
			// stop the worker.
			W.Stop();
			GoButton.Enabled = true;
			StopButton.Enabled = false;
			OutputBox.AppendText("Thread " + WorkerThread.Name + " Stopped.\n");
		}
	}
}
